#!/usr/bin/env python
import setpath
import os
import unittest
from bike.testutils import *
from bike.parsing.extended_ast import Root, getRoot
from bike.parsing.newstuff import getModule, getModuleOrPackageUsingFQN,\
     generateModuleFilenamesInPythonPath, getSourceNodesContainingRegex,\
     getRegexMatchesInPythonPath
from bike.query.getTypeOf import getTypeOf

class TestGetModule(BRMTestCase):

    def test_getRootWorksAfterDefinedByCreateSourceNodeAt(self):
        src=trimLines("""
        class TheClass:
            pass
        a = TheClass()
        """)
        root = createSourceNodeAt_old(Root(),src,"mymodule")
        assert root == getRoot()

    ''' no longer relevant
    def test_getsModuleDefinedByCreateSourceNodeAt(self):
        src=trimLines("""
        class TheClass:
            pass
        a = TheClass()
        """)
        root = createSourceNodeAt_old(Root(),src,"mymodule")
        m = getModule(os.path.abspath("mymodule.py"))
        self.assertEqual(m,getTypeOf(root,"mymodule"))
    '''

    def test_returnsNoneIfModuleDoesntExist(self):
        assert getModule(tmpfile) == None
        

class TestGetModuleOrPackageUsingFQN(BRMTestCase):
    def test_worksForFullPath(self):
        try:
            createPackageStructure("pass","pass")
            Root(None,None,[pkgstructureRootDir])
            self.assertEqual(getModuleOrPackageUsingFQN("a.b.bah").filename,
                             pkgstructureFile2)
        finally:
            removePackageStructure()
        
    def test_worksForPackage(self):
        try:
            createPackageStructure("pass","pass")
            Root(None,None,[pkgstructureRootDir])
            self.assertEqual(getModuleOrPackageUsingFQN("a.b").path,
                             pkgstructureChilddir)
        finally:
            removePackageStructure()

    '''
    def test_worksForRoot(self):
        try:
            os.makedirs(pkgstructureRootDir)
            writeFile(os.path.join(pkgstructureRootDir,"foo.py"),"#")
            r = Root(None,None,[pkgstructureRootDir])
            self.assertEqual(getModuleOrPackageUsingFQN("foo.py",r).path)

        finally:
            os.remove(os.path.join(pkgstructureRootDir,"foo.py"))
            os.removedirs(pkgstructureRootDir)
    '''

class TestGenerateModuleFilenamesInPythonPath(BRMTestCase):
    def test_works(self):
        try:
            createPackageStructure("pass","pass")
            fnames = [f for f in \
                 generateModuleFilenamesInPythonPath([pkgstructureRootDir])]


            assert os.path.join(pkgstructureBasedir,"__init__.py") in fnames
            assert pkgstructureFile1 in fnames
            assert os.path.join(pkgstructureChilddir,"__init__.py") in fnames
            assert pkgstructureFile2 in fnames
            assert len(fnames) == 4
        finally:
            removePackageStructure()

    def test_doesntTraverseIntoNonPackages(self):
        try:
            createPackageStructure("pass","pass")
            nonPkgDir = os.path.join(pkgstructureChilddir,"c")
            newfile = os.path.join(nonPkgDir,"baz.py")
            # N.B. don't put an __init__.py in it, so isnt a package
            os.makedirs(nonPkgDir)
            writeFile(newfile,"pass")
            fnames = [f for f in \
                 generateModuleFilenamesInPythonPath([pkgstructureRootDir])]
            assert newfile not in fnames
        finally:
            #os.remove(initfile)
            os.remove(newfile)
            os.removedirs(nonPkgDir)
            removePackageStructure()
        

    def test_doesntTraverseIntoNonPackages(self):
        try:
            createPackageStructure("pass","pass")
            nonPkgDir = os.path.join(pkgstructureChilddir,"c")
            newfile = os.path.join(nonPkgDir,"baz.py")
            # N.B. don't put an __init__.py in it, so isnt a package
            os.makedirs(nonPkgDir)
            writeFile(newfile,"pass")
            fnames = [f for f in \
                 generateModuleFilenamesInPythonPath([pkgstructureRootDir])]
            assert newfile not in fnames
        finally:
            #os.remove(initfile)
            os.remove(newfile)
            os.removedirs(nonPkgDir)
            removePackageStructure()

    def test_doesntTraverseIntoNonPackagesUnderRoot(self):
        try:
            os.makedirs(pkgstructureBasedir)
            writeFile(pkgstructureFile1,"pass")
            fnames = [f for f in \
                      generateModuleFilenamesInPythonPath([pkgstructureRootDir])]
            assert pkgstructureFile1 not in fnames
        finally:
            os.remove(pkgstructureFile1)
            os.removedirs(pkgstructureBasedir)


class TestGetSourceNodesContainingRegex(BRMTestCase):
    def test_works(self):
        try:
            createPackageStructure("# testregexfoobah","pass")
            Root(None,None,[pkgstructureRootDir])
            srcfiles = [s for s in 
                        getSourceNodesContainingRegex("testregexfoobah")]
            self.assertEqual(pkgstructureFile1,srcfiles[0].filename)
        finally:
            removePackageStructure()
        
class TestGetRegexMatchesInPythonPath(BRMTestCase):
    def test_works(self):

        src = '''# testregexfoobah
        foobahtestarseba
        '''
        regex = "test...."
        res = self.helper(src,regex)
        self.assertEqual(pkgstructureFile1,res[0][0])
        self.assertEqual("testrege",res[0][1])
        self.assertEqual(pkgstructureFile1,res[1][0])
        self.assertEqual("testarse",res[1][1])
        self.assertEqual(2,len(res))

    def helper(self,src,regex):
        try:
            createPackageStructure(src,"pass")
            res = [s for s in 
                        getRegexMatchesInPythonPath(regex)]
        finally:
            removePackageStructure()
        return res

if __name__ == "__main__":
    unittest.main()
